function [sys,x0,str,ts] = NLdyn(t,x,u,flag,incond,Htot,Mtot1,Jtot,Rwheel)
% CSFUNC An example M-file S-function for defining a system of
% continuous state equations:
%      x' = Ax + Bu
%      y  = Cx + Du

%
% Dispatch the flag.
%
switch flag

  case 0
    [sys,x0,str,ts]=mdlInitializeSizes(incond); % Initialization

  case 1
    sys = mdlDerivatives(t,x,u,Htot,Mtot1,Jtot,Rwheel); % Calculate derivatives

  case 3
    sys = mdlOutputs(t,x,u); % Calculate outputs
  
  case { 2, 4, 9 } % Unused flags
    sys = [];
  otherwise
    error(['Unhandled flag = ',num2str(flag)]); % Error handling
end
% End of csfunc.
%==============================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the 
% S-function.
%==============================================================
%
function [sys,x0,str,ts] = mdlInitializeSizes(incond)
%
% Call simsizes for a sizes structure, fill it in and convert it 
% to a sizes array.
%

sizes = simsizes;
sizes.NumContStates  = 3;
sizes.NumDiscStates  = 0;
sizes.NumOutputs     = 3;
sizes.NumInputs      = 1;
sizes.DirFeedthrough = 0;
sizes.NumSampleTimes = 1;

sys = simsizes(sizes);
%
% Initialize the initial conditions.
%
x0 = incond;
%
% str is an empty matrix.
%
str = [];
%
% Initialize the array of sample times; in this example the sample 
% time is continuous, so set ts to 0 and its offset to 0.
%
ts = [0 0];
% End of mdlInitializeSizes.
%
%==============================================================
% mdlDerivatives
% Return the derivatives for the continuous states.
%==============================================================
function sys = mdlDerivatives(t,x,u,Htot,Mtot1,Jtot,Rwheel)

g=9.80665;

X1=1/(Mtot1*Jtot-Htot^2*(cos(x(2)))^2);

A=[0    0   X1*Jtot*Htot*sin(x(2))*x(3)
    0   0   1
    0   0   -X1*Htot^2*sin(x(2))*cos(x(2))*x(3)];

B1=X1*[-Htot^2*g*cos(x(2))*sin(x(2))
   0
   Htot*Mtot1*g*sin(x(2))];

B2=2*X1*[Jtot/Rwheel+Htot*cos(x(2))
        0
        -Mtot1-Htot/Rwheel*cos(x(2))];

xdot= A*x + B1 + B2*u;


sys = xdot;
% End of mdlDerivatives.
%
%==============================================================
% mdlOutputs
% Return the block outputs.
%==============================================================
%
function sys = mdlOutputs(t,x,u)

y=x;

sys = y;

% End of mdlOutputs.
