% physical and geometrical data
% values are in [m], [kg] and derivates

g=9.80665;
alfa=0;

% Body
Mbody=7.78;             % mass
dbody=62.56/1000;       % distance of C.o.G. from wheel axis
Jbody=28291.779/(1e6);  % Inertia at C.o.G. along Y axis (parallel to wheels)

% Wheel
Mwheel=0.56723;
Jwheel=1570.2658/1e6;
Rwheel=150/1000/2;      % wheel radius

% electronic devices
Med=0.272886;
ded=(140+27.5)/1000;
Jed=525.87421/1e6;

% pole
Mpole=0.49647913;
dpole=(140+500)/1e3;
Jpole=41393.541/1e6;

% horizontal item1
Mhi=0.097806389;
dhi1=(140+500)/1e3;
Jhi=7.9904629/1e6;


% horizontal item2
dhi2=(140+990)/1e3;


% Payload
% Mpl=2.355;
% dpl=(140+1000+25)/1e3;
% Jpl=2453.125/1e6;
Mpl=0;
dpl=0;
Jpl=0;


% overall masses and inertias

Mtot=Mbody+2*Mwheel+Med+2*Mpole+2*Mhi+Mpl;
Mtot1=Mtot+2*Jwheel/Rwheel^2;

Vmass=[Mbody Med 2*Mpole Mhi Mhi Mpl];
Vd=[dbody ded dpole dhi1 dhi2 dpl];

Htot=Vmass*Vd';

Jtot=Vmass*(Vd.^2)'+Jbody+Jed+2*Jpole+2*Jhi+Jpl;

% fuzzy model

th=linspace(-pi/2,pi/2,1000);

% estimation of maximum angular velocity
omMax=sqrt(Mtot*g*dpole/Jtot);


% Motor Parameters

Rmot=2.36; % [ohm]
Lmot=0.199e-3;  % [henry]

Kt=25.8e-3; % [Nm/A]
Ke=Kt;
Valim=24; % [V]

% function z1 is in z1.m file

% % vector containing minimum and maximum values of z1
% cz1=[z1(pi/2,Htot,Mtot1,Jtot) z1(0,Htot,Mtot1,Jtot)];
% 
% % vector containing minimum and maximum values of z2
% cz2=[1 0];
% 
% % vector containing minimum and maximum values of z3
% cz3=[omMax -omMax];
% 
% % vector containing minimum and maximum values of z4
% cz4=[1 2/pi];
% 
% Atot=[];
% Btot=[];
% v_indexA=[];
% v_indexB=[];
% 
% for l=1:2
%     for k=1:2
%         for j=1:2
%             for i=1:2
%                 v_indexA=[v_indexA;i j k l];
%                 Atemp=cz1(i)*Htot*[0 Htot*g*cz2(j)*cz4(l) -Jtot*cz3(k)
%                                   0           0             1
%                                   0 -Mtot1*g*cz4(l)      Htot*cz2(j)*cz3(k)];
%                 Atot=[Atot;Atemp];
%             end
%         end
%     end
% end
% 
% for l=1:2
%     for k=1:2
%         for j=1:2
%             for i=1:2
% %                v_indexB=[v_indexB;i j k l];
%                 Btemp=2*cz1(i)*[-Htot*cz2(j)-Jtot/Rwheel; 0 ; Htot/Rwheel*cz2(j)+Mtot1];
%                 Btot=[Btot;Btemp];
%             end
%         end
%     end
% end

% v_indexA
% v_indexB
% Atot
% Btot



% Simulink parameters

stepc=0.02;
tau=33;               % rapporto di riduzione per GP 32A
passi=1024;           % numero di passi al giro per l'encoder
om_max=0.2;            % velocit massima letta dal gyro (corrisponde a 5V)
om_min=-0.2;           % velocit minima letta dal gyro (corrisponde a 0V)
bit_adxl=2^10;        % numero di bit disponibili per rappresentare la
                      % lettura dell'accelerometro ADXL 220
res_C=0.01;
res_acc=9.8/140;

Cmax=7.6;             % Coppia max disponibile alle ruote


errang=0;           % errore di lettura dell'angolo in gradi

Cmat=eye(3);
Dmat=zeros(3,1);


% CONDIZIONI INIZIALI DEL ROBOT

incond=[0 0 0]';

% ERRORI DI LETTURA

P_x=1e-6;           % rumore encoder
P_th=1e-6;          % rumore adxl

fil_x=10;           % frequenza di filtraggio encoder
fil_th=20;           % frequenza di filtraggio angolo


% COPPIA

Torque=1/2*Mtot1*g*sin(alfa)*Rwheel;      % coppia statica



%%%%%%%%%%%%%%%%%%%
% Linearized model
%%%%%%%%%%%%%%%%%%%

Qlin=[0 1 0; Mtot 0 Htot; Htot 0 Jtot];

Atemp=[0 0 1; 0 0 0; 0 Htot*g 0];

Btemp=[0; 2/Rwheel; -2];

Alin=inv(Qlin)*Atemp;
Blin=inv(Qlin)*Btemp;

disp(['Eigenvalues: ',num2str((eig(Alin))')])

% observability
C2=[1 0 0 ;0 0 1];
N=[C2' Alin'*C2' (Alin')^2*C2'];
disp(['dynamical observability: ',num2str(rank(N))])


% LQR

P=diag([1 10 100]);
Rmax=0.1;

[Kc1,Sc1,Ec1]=lqr(Alin,Blin,P,Rmax)

[Kc2,Sc2,Ec2]=lqrd(Alin,Blin,P,Rmax,stepc)

K=Kc2

% Matrici per osservatore ridotto

A11=Alin(1:2,1:2);
A12=Alin(1:2,3);
A21=Alin(3,1:2);
A22=Alin(3,3);

B1=[Blin(1:2)];
B2=[Blin(3)];

om1=-10;

LT=place(A22',A12',om1);

L=LT'
F=A22-L*A12
G=B2-L*B1
H=A21+F*L-L*A11

[Ao, Bo, Co, Do] = c2dm(F,[G,H],eye(1),zeros(1,3),stepc,'zoh')

% discretizzazione sistema
% [Aod, Bod, Cod, Dod] = c2dm(Alin,Blin,Cmat,Dmat,stepc,'zoh')
% 
% A11=Aod(1:2,1:2);
% A12=Aod(1:2,3);
% A21=Aod(3,1:2);
% A22=Aod(3,3);
% 
% B1=[Bod(1:2)];
% B2=[Bod(3)];
% 
% om1=-4;
% 
% LT=place(A22',A12',om1);
% 
% L=LT'
% F=A22-L*A12
% G=B2-L*B1
% H=A21+F*L-L*A11
% 
% Ao = F;
% Bo = [G,H];
% Co = eye(1);
% Do = zeros(1,3);


% matrici per il fuzzy e calcoli LMI

% linearizziamo il sistema in theta = 0
X1f0 = 1/(Mtot1*Jtot-Htot^2);

Af0 = [0 -X1f0*Htot^2*g 0
    0 0 1
    0 X1f0*Htot*Mtot1*g 0];

Bf0 = [2*X1f0*(Jtot/Rwheel+Htot)
    0
    -2*X1f0*(Mtot1+Htot/Rwheel)];

% discretizzazione matrici
[Af0d, Bf0d, C, D] = c2dm(Af0,Bf0,Cmat,Dmat,stepc,'zoh')

% linearizziamo il sistema in theta = +-pi/4
X1f1 = 1/(Mtot1*Jtot-Htot^2/2);

Af1 = [0 -X1f1*Htot^2*g*2/pi 0
    0 0 1
    0 X1f1*Htot*Mtot1*g*2*sqrt(2)/pi 0];

Bf1 = [2*X1f1*(Jtot/Rwheel+Htot*sqrt(2)/2)
    0
    -2*X1f1*(Mtot1+Htot*sqrt(2)/(2*Rwheel))];

% discretizzazione matrici
[Af1d, Bf1d, C, D] = c2dm(Af1,Bf1,Cmat,Dmat,stepc,'zoh')

setlmis([]);

% lmivar:
% primo input => 1=tipo=matrice X simmetrica diagonale a blocchi;
% secondo input => matrice: [dimensione 1|0|-1] di cui
%                               1 => full simmetric block
%                               0 => scalar block
%                              -1 => zero block
X=lmivar(1,[3 1]);

M1 = lmivar(2,[1 3]);
M2 = lmivar(2,[1 3]);

% for each rule
% il riferimento  X < 0 (j). Quindi se devo scrivere X > 0 metto -j

% per M1
    lmiterm([-1 1 1 X],1,1);                % LMI #1: X
    lmiterm([-1 2 1 X],Af0d,1);             % LMI #1: A0fd*X
    lmiterm([-1 2 1 M1],Bf0d,-1);           % LMI #1: -B0fd*M1
    lmiterm([-1 2 2 X],1,1);                % LMI #1: X

% per M2
    lmiterm([-2 1 1 X],1,1);                % LMI #1: X
    lmiterm([-2 2 1 X],Af1d,1);             % LMI #1: A1fd*X
    lmiterm([-2 2 1 M2],Bf1d,-1);           % LMI #1: -B1fd*M2
    lmiterm([-2 2 2 X],1,1);                % LMI #1: X

% termine misto
    
lmiterm([-3 1 1 X],1,1);                    % LMI #3: X
lmiterm([-3 2 1 X],Af0d/2,1);
lmiterm([-3 2 1 X],Af1d/2,1);
lmiterm([-3 2 1 M2],-Bf0d/2,1);
lmiterm([-3 2 1 M1],-Bf1d/2,1);
lmiterm([-3 2 2 X],1,1);                    % LMI #3: X


lmiterm([-4 1 1 X],1,1);                    % LMI #4: X

tiltylmi=getlmis;
  
[copt xopt]=feasp(tiltylmi);
  
Xopt=dec2mat(tiltylmi,xopt,X);

M1opt = dec2mat(tiltylmi,xopt,M1);
M2opt = dec2mat(tiltylmi,xopt,M2);

F1opt = M1opt * inv(Xopt);
F2opt = M2opt * inv(Xopt);

tilty = readfis('tilty');
tilty.impMethod = 'prod';
tilty.output.mf(1).params = [F1opt 0];
tilty.output.mf(2).params = [F2opt 0];

%osservatore ridotto fuzzy

%prima regola (partendo dalle matrici A0f e B0f)

Af011=Af0(1:2,1:2);
Af012=Af0(1:2,3);
Af021=Af0(3,1:2);
Af022=Af0(3,3);

Bf01=[Bf0(1:2)];
Bf02=[Bf0(3)];

om1=-10;

LT=place(Af022',Af012',om1);

L0=LT'
F=Af022-L*Af012
G=Bf02-L*Bf01
H=Af021+F*L-L*Af011

[Afo0, Bfo0, Cfo0, Dfo0] = c2dm(F,[G,H],eye(1),zeros(1,3),stepc,'zoh');

%prima regola (partendo dalle matrici A1f e B1f)

Af111=Af1(1:2,1:2);
Af112=Af1(1:2,3);
Af121=Af1(3,1:2);
Af122=Af1(3,3);

Bf11=[Bf1(1:2)];
Bf12=[Bf1(3)];

om1=-10;

LT=place(Af122',Af112',om1);

L1=LT'
F=Af122-L*Af112
G=Bf12-L*Bf11
H=Af121+F*L-L*Af111

[Afo1, Bfo1, Cfo1, Dfo1] = c2dm(F,[G,H],eye(1),zeros(1,3),stepc,'zoh')