% il sistema  il solito cart-pole. Il cart  costituito dalle ruote e dai
% motori, il pole  costituito da tutto il resto della struttura (il corpo
% del robot, le batterie e l'eventuale carico).

% le seguenti variabili sono globali
%function [M, m, J, d, g, r, A, B, C, D] = model()

% C O S T A N T I

    g = 9.80665;    % accelerazione gravitazionale
    h = 0.001;      % coefficiente di attrito viscoso [Kg/s]

% P A R A M E T R I   C A R T

    Mruote = 0.6*2;         % massa delle ruote [Kg]
    Mmotori = 0.3*2;        % massa dei motori [Kg]
    
    r = 0.075;              % raggio della ruota [m]
    
    % momenti d'inerzia
    jRuote = 1/2 * Mruote * r^2;              % momento d'inerzia delle ruote
    jMotori = 0; % momento d'inerzia dei motori G U A R D A R E   I L   D A T A S H E E T
    
    M = Mruote + Mmotori;       % massa totale del cart [Kg]
    
% P A R A M E T R I   D E L   P O L E

    Mcestello = 1.925;   % massa del cestello portabatterie [Kg]
    %dove:
    %0.251 + 0.093 + 0.158*2 + 0.060 + 0.075 + 0.026 + 0.115*2 + 0.474 + 0.4 = 1.925; [Kg]
    %poggia batterie = 0.251 [Kg]
    %griglia ripara motori = 0.093 [Kg]
    %laterale esterno = 0.158 [Kg]
    %portello batterie = 0.060 [Kg]
    %barra fermaportello = 0.075 [Kg]
    %barra fermabatterie = 0.026 [Kg]
    %laterale interno = 0.115 [Kg]
    %scheda con supporto = 0.474
    %lato superiore = 0.4 [Kg]
    Mbatterie = 2.4*2;                      % massa delle batterie [Kg]
    Mstruttura = 1.4;                       % massa della struttura [Kg]
    Mcarico = 0;                            % massa del carico [Kg], 0 perch per ora non c'

    % dimensioni dei componenti del pole
    % cestello
    ac = 0.165;                             % larghezza del cestello [m]
    bc = 0.14;                              % altezza del cestello [m]
    % batterie
    ab = 0.15;                              % larghezza batterie [m]
    bb = 0.095;                             % altezza batterie [m]
    % struttura
    Lstruttura = 0.865;                     % lunghezza della struttura [m]
    % carico
    acarico = 0.02;                            % larghezza del carico [m]
    bcarico = 0.02;                            % altezza del carico [m]
    
    % distanze dei centri di massa dei componenti dall'asse di rotazione delle ruote.
    dcestello = bc/2;                       % per il cestello [m]
    dbatterie = bb/2 + 0.018;               % per le batterie [m]
    dstruttura = Lstruttura/2 + 0.14;       % per la struttura [m]
    dcarico = acarico/2 + 0.14 + Lstruttura;                            % per il carico [m], 0 perch per ora non c'

    % momenti d'inerzia
    jcestello = Mcestello * (1/2 * (ac^2 + bc^2) + dcestello^2);        % momento d'inerzia del cestello
    jbatterie = Mbatterie * (1/2 * (ab^2 + bb^2) + dbatterie^2);        % momento d'inerzia delle batterie
    jstruttura = Mstruttura * (1/12 * (Lstruttura/2)^2 + dstruttura^2); % momento d'inerzia della struttura
    jcarico = Mcarico * (1/2 * (acarico^2 + bcarico^2) + dcarico^2);    % momento d'inerzia del carico
    
    % massa totale del pole
    m = Mcestello + Mbatterie + Mstruttura + Mcarico;                   % massa totale del pole
    
    % centro di massa del pole
    d = (Mcestello*dcestello + Mbatterie*dbatterie + Mstruttura*dstruttura + Mcarico*dcarico) / m;  % centro di massa del pole
    
    % momento d'inerzia totale del pole rispetto all'asse delle ruote
    J = jcestello + jbatterie + jstruttura + jcarico;
    
    
    % MATRICI DEL MODELLO
    % x' = Ax + Bu
    % y  = Cx + Du
    
    % variabili di appoggio
    den = (M+m)*(J+m*d^2)-m^2*d^2;
    
    % matrice A
        A = zeros(3,3);
        A(1,1) = -(J + m * d^2) * h / den;
        A(1,2) = m^2 * g * d^2 / den;
        A(2,3) = 1;
        A(3,1) = m * d * h / den;
        A(3,2) = - m * g * d * (M + m) / den;

    % matrice B
        B = zeros(3,1);
        B(1) = (J + m * d^2) / (den*r);
        B(3) = - m * d / (den*r);
    
    % matrice C
        C = eye(3);
    
    % matrice D
        D = zeros(3,1);